searchData={"items":[{"type":"extras","title":"Changelog","doc":"# Changelog\n\n## [Unreleased]","ref":"changelog.html"},{"type":"extras","title":"v0.1.0 [2024-09-11] - Changelog","doc":"- Initial release.","ref":"changelog.html#v0-1-0-2024-09-11"},{"type":"extras","title":"Overview","doc":"# brod_oauth\n\n[![Build Status](https://github.com/HCA-Healthcare/brod_oauth/actions/workflows/ci.yml/badge.svg)](https://github.com/HCA-Healthcare/brod_oauth/actions/workflows/ci.yml)\n[![Hex pm](https://img.shields.io/hexpm/v/brod_oauth.svg)](https://hex.pm/packages/brod_oauth)\n[![Docs](https://img.shields.io/badge/hex-docs-green.svg?style=flat)](https://hexdocs.pm/brod_oauth)\n[![Erlang Versions](https://img.shields.io/badge/Supported%20Erlang%2FOTP-25.0%20to%2027.0-blue)](http://www.erlang.org)\n\n`brod_oauth` is a plugin for [brod](https://github.com/kafka4beam/brod) which adds SASL/OAUTHBEARER auth support.","ref":"readme.html"},{"type":"extras","title":"Setup - Overview","doc":"Add `brod_oauth` to your deps in rebar.config or mix.exs :","ref":"readme.html#setup"},{"type":"extras","title":"rebar.config - Overview","doc":"```erlang\n{deps, [{brod_oauth, \"0.1.0\"}]}.\n```","ref":"readme.html#rebar-config"},{"type":"extras","title":"mix.exs - Overview","doc":"```elixir\n{:brod_oauth, \"~> 0.1.0\"}\n```\n\n**NOTE** : `brod_oauth` requires `kafka_protocol` >= `4.1.8`.","ref":"readme.html#mix-exs"},{"type":"extras","title":"Configuration and usage - Overview","doc":"The only required configuration for `brod` is a sasl callback tuple with specific configuration\nfor `brod_oauth` enclosed in a map. See [brod authentication\nsupport](https://github.com/kafka4beam/brod?tab=readme-ov-file#authentication-support)\nfor more general information on authentication support in brod.\n\n\n#### brod_oauth configuration map\n\n**Required keys** :\n\n- `token_callback` : An arity 1 callback function. Said function is given a map containing\nconnection properties and expected to return a map containing the key\n`token`, pointing to a valid JWT retrieved from an oauth provider.\nThe current properties passed to the callback are as follows :\n     - `client_id` : The client id of the brod client instance.\n     - `host` : Hostname for the current connection requiring auth.\n     - `timeout` : The timeout value associated with connection (i.e., `connect_timeout`).\n\nOptional keys :\n\n- `authz_id` : A utf-8 binary authz id value.\n- `extensions` : A map of key value pairs, most commonly a logical cluster id and identity pool id.\n\n#### Examples\n\nNote : While the examples below demonstrate providing `extensions` in configuration, it is not a\nrequired parameter, and not all setups require extensions to be provided. Such is the case with\n`authz_id` as well.\n\n##### Erlang\n\n```erlang\n-module(example).\n\n-export([start_client/0, fetch_token/1]).\n\nfetch_token(#{client_id := _, host := _, timeout := _) ->\n\n    BodyParams = [\n        {\"grant_type\", \"grant_type\"},\n        {\"client_secret\", \"client_secret\"},\n        {\"client_id\", \"client_id\"},\n        {\"scope\", \"scope\"}\n    ],\n\n    Body = uri_string:compose_query(BodyParams1),\n\n    Res = httpc:request(\n        post,\n        {\"https://my.oauth.provider\", [], \"application/x-www-form-urlencoded\", Body},\n        [],\n        [{body_format, binary}]\n    ),\n\n    case Res of\n        {ok, {{\"HTTP/1.1\", 200, \"OK\"}, _headers, Json}} ->\n            #{<<\"access_token\">> := Token} = json:decode(Json),\n            {ok, #{\n                token => Token\n            }};\n        _Err ->\n            {error, failed_to_obtain_jwt}\n    end.\n\nstart_client() ->\n  BootstrapEndpoints = [{\"my.kafka.broker.host\", 9092}],\n  Extensions = #{\n                  <<\"logicalCluster\">> => <<\"lkc-1234\">>,\n                  <<\"identityPoolId\">> => <<\"pool-4321\">>},\n  BrodOauthConfig = #{token_callback => fun example:fetch_token/1, extensions => Extensions},\n  BrodConfig = [{connect_timeout, 60000},\n                {sasl, {callback, brod_oauth, BrodOauthConfig}}],\n  ok = brod:start_client(BootstrapEndpoints, my_client, BrodConfig).\n```\n\n##### Elixir\n\n```elixir\n\ndefmodule Example do\n\n  def oauth_params(%{client_id: _, host: _,  timeout: _) do\n    body_params = %{\n      \"grant_type\" => \"grant_type\",\n      \"client_secret\" => \"client_secret\",\n      \"client_id\" => \"client_id\",\n      \"scope\" => \"scope\"\n    }\n\n    body = URI.encode_query(body_params1)\n\n    res =\n      :httpc.request(\n        :post,\n        {to_charlist(oauth_config.url), [], ~c\"application/x-www-form-urlencoded\",\n         to_charlist(body)},\n        [],\n        [{:body_format, :binary}]\n      )\n\n    case res do\n      {:ok, {{~c\"HTTP/1.1\", 200, ~c\"OK\"}, _headers, json}} ->\n        %{\"access_token\" => token} = :json.decode(json)\n\n        {:ok,\n         %{\n           token: token,\n           extensions: extensions\n         }}\n\n      _ ->\n        {:error, :failed_to_obtain_jwt}\n    end\n  end\n\n  def start_client() do\n    bootstrap_endpoints = [{\"my.kafka.broker.host\", 9092}]\n    extensions = %{\"logicalCluster\" => \"lkc-1234\", \"identityPoolId\" => \"pool-4321\"}\n    brod_oauth_config = %{token_callback: &Example:fetch_token/1, extensions: extensions}\n    brod_config = [connect_timeout: 60000, sasl: {:callback, :brod_oauth, brod_oauth_config}}]\n    :ok = :brod.start_client(bootstrap_endpoints, :my_client, brod_config)\n  end\n```\n\nSee the [examples](https://github.com/HCA-Healthcare/brod_oauth/tree/main/examples) for more detail.\n\n# Further reading\n\n- [Kafka KIP-255](https://cwiki.apache.org/confluence/pages/viewpage.action?pageId=75968876)\n- [Kafka KIP-768](https://cwiki.apache.org/confluence/pages/viewpage.action?pageId=186877575)\n- [RFC-5801](https://www.rfc-editor.org/rfc/rfc5801.html)","ref":"readme.html#configuration-and-usage"},{"type":"extras","title":"License","doc":"Apache License\n==============\n\n_Version 2.0, January 2004_\n_&lt; &gt;_","ref":"license.html"},{"type":"extras","title":"Terms and Conditions for use, reproduction, and distribution - License","doc":"#### 1. Definitions\n\n“License” shall mean the terms and conditions for use, reproduction, and\ndistribution as defined by Sections 1 through 9 of this document.\n\n“Licensor” shall mean the copyright owner or entity authorized by the copyright\nowner that is granting the License.\n\n“Legal Entity” shall mean the union of the acting entity and all other entities\nthat control, are controlled by, or are under common control with that entity.\nFor the purposes of this definition, “control” means **(i)** the power, direct or\nindirect, to cause the direction or management of such entity, whether by\ncontract or otherwise, or **(ii)** ownership of fifty percent (50%) or more of the\noutstanding shares, or **(iii)** beneficial ownership of such entity.\n\n“You” (or “Your”) shall mean an individual or Legal Entity exercising\npermissions granted by this License.\n\n“Source” form shall mean the preferred form for making modifications, including\nbut not limited to software source code, documentation source, and configuration\nfiles.\n\n“Object” form shall mean any form resulting from mechanical transformation or\ntranslation of a Source form, including but not limited to compiled object code,\ngenerated documentation, and conversions to other media types.\n\n“Work” shall mean the work of authorship, whether in Source or Object form, made\navailable under the License, as indicated by a copyright notice that is included\nin or attached to the work (an example is provided in the Appendix below).\n\n“Derivative Works” shall mean any work, whether in Source or Object form, that\nis based on (or derived from) the Work and for which the editorial revisions,\nannotations, elaborations, or other modifications represent, as a whole, an\noriginal work of authorship. For the purposes of this License, Derivative Works\nshall not include works that remain separable from, or merely link (or bind by\nname) to the interfaces of, the Work and Derivative Works thereof.\n\n“Contribution” shall mean any work of authorship, including the original version\nof the Work and any modifications or additions to that Work or Derivative Works\nthereof, that is intentionally submitted to Licensor for inclusion in the Work\nby the copyright owner or by an individual or Legal Entity authorized to submit\non behalf of the copyright owner. For the purposes of this definition,\n“submitted” means any form of electronic, verbal, or written communication sent\nto the Licensor or its representatives, including but not limited to\ncommunication on electronic mailing lists, source code control systems, and\nissue tracking systems that are managed by, or on behalf of, the Licensor for\nthe purpose of discussing and improving the Work, but excluding communication\nthat is conspicuously marked or otherwise designated in writing by the copyright\nowner as “Not a Contribution.”\n\n“Contributor” shall mean Licensor and any individual or Legal Entity on behalf\nof whom a Contribution has been received by Licensor and subsequently\nincorporated within the Work.\n\n#### 2. Grant of Copyright License\n\nSubject to the terms and conditions of this License, each Contributor hereby\ngrants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free,\nirrevocable copyright license to reproduce, prepare Derivative Works of,\npublicly display, publicly perform, sublicense, and distribute the Work and such\nDerivative Works in Source or Object form.\n\n#### 3. Grant of Patent License\n\nSubject to the terms and conditions of this License, each Contributor hereby\ngrants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free,\nirrevocable (except as stated in this section) patent license to make, have\nmade, use, offer to sell, sell, import, and otherwise transfer the Work, where\nsuch license applies only to those patent claims licensable by such Contributor\nthat are necessarily infringed by their Contribution(s) alone or by combination\nof their Contribution(s) with the Work to which such Contribution(s) was\nsubmitted. If You institute patent litigation against any entity (including a\ncross-claim or counterclaim in a lawsuit) alleging that the Work or a\nContribution incorporated within the Work constitutes direct or contributory\npatent infringement, then any patent licenses granted to You under this License\nfor that Work shall terminate as of the date such litigation is filed.\n\n#### 4. Redistribution\n\nYou may reproduce and distribute copies of the Work or Derivative Works thereof\nin any medium, with or without modifications, and in Source or Object form,\nprovided that You meet the following conditions:\n\n* **(a)** You must give any other recipients of the Work or Derivative Works a copy of\nthis License; and\n* **(b)** You must cause any modified files to carry prominent notices stating that You\nchanged the files; and\n* **(c)** You must retain, in the Source form of any Derivative Works that You distribute,\nall copyright, patent, trademark, and attribution notices from the Source form\nof the Work, excluding those notices that do not pertain to any part of the\nDerivative Works; and\n* **(d)** If the Work includes a “NOTICE” text file as part of its distribution, then any\nDerivative Works that You distribute must include a readable copy of the\nattribution notices contained within such NOTICE file, excluding those notices\nthat do not pertain to any part of the Derivative Works, in at least one of the\nfollowing places: within a NOTICE text file distributed as part of the\nDerivative Works; within the Source form or documentation, if provided along\nwith the Derivative Works; or, within a display generated by the Derivative\nWorks, if and wherever such third-party notices normally appear. The contents of\nthe NOTICE file are for informational purposes only and do not modify the\nLicense. You may add Your own attribution notices within Derivative Works that\nYou distribute, alongside or as an addendum to the NOTICE text from the Work,\nprovided that such additional attribution notices cannot be construed as\nmodifying the License.\n\nYou may add Your own copyright statement to Your modifications and may provide\nadditional or different license terms and conditions for use, reproduction, or\ndistribution of Your modifications, or for any such Derivative Works as a whole,\nprovided Your use, reproduction, and distribution of the Work otherwise complies\nwith the conditions stated in this License.\n\n#### 5. Submission of Contributions\n\nUnless You explicitly state otherwise, any Contribution intentionally submitted\nfor inclusion in the Work by You to the Licensor shall be under the terms and\nconditions of this License, without any additional terms or conditions.\nNotwithstanding the above, nothing herein shall supersede or modify the terms of\nany separate license agreement you may have executed with Licensor regarding\nsuch Contributions.\n\n#### 6. Trademarks\n\nThis License does not grant permission to use the trade names, trademarks,\nservice marks, or product names of the Licensor, except as required for\nreasonable and customary use in describing the origin of the Work and\nreproducing the content of the NOTICE file.\n\n#### 7. Disclaimer of Warranty\n\nUnless required by applicable law or agreed to in writing, Licensor provides the\nWork (and each Contributor provides its Contributions) on an “AS IS” BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied,\nincluding, without limitation, any warranties or conditions of TITLE,\nNON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are\nsolely responsible for determining the appropriateness of using or\nredistributing the Work and assume any risks associated with Your exercise of\npermissions under this License.\n\n#### 8. Limitation of Liability\n\nIn no event and under no legal theory, whether in tort (including negligence),\ncontract, or otherwise, unless required by applicable law (such as deliberate\nand grossly negligent acts) or agreed to in writing, shall any Contributor be\nliable to You for damages, including any direct, indirect, special, incidental,\nor consequential damages of any character arising as a result of this License or\nout of the use or inability to use the Work (including but not limited to\ndamages for loss of goodwill, work stoppage, computer failure or malfunction, or\nany and all other commercial damages or losses), even if such Contributor has\nbeen advised of the possibility of such damages.\n\n#### 9. Accepting Warranty or Additional Liability\n\nWhile redistributing the Work or Derivative Works thereof, You may choose to\noffer, and charge a fee for, acceptance of support, warranty, indemnity, or\nother liability obligations and/or rights consistent with this License. However,\nin accepting such obligations, You may act only on Your own behalf and on Your\nsole responsibility, not on behalf of any other Contributor, and only if You\nagree to indemnify, defend, and hold each Contributor harmless for any liability\nincurred by, or claims asserted against, such Contributor by reason of your\naccepting any such warranty or additional liability.\n\n_END OF TERMS AND CONDITIONS_","ref":"license.html#terms-and-conditions-for-use-reproduction-and-distribution"},{"type":"extras","title":"APPENDIX: How to apply the Apache License to your work - License","doc":"To apply the Apache License to your work, attach the following boilerplate\nnotice, with the fields enclosed by brackets `[]` replaced with your own\nidentifying information. (Don't include the brackets!) The text should be\nenclosed in the appropriate comment syntax for the file format. We also\nrecommend that a file or class name and description of purpose be included on\nthe same “printed page” as the copyright notice for easier identification within\nthird-party archives.\n\n    Copyright [yyyy] [name of copyright owner]\n\n    Licensed under the Apache License, Version 2.0 (the \"License\");\n    you may not use this file except in compliance with the License.\n    You may obtain a copy of the License at\n\n      http://www.apache.org/licenses/LICENSE-2.0\n\n    Unless required by applicable law or agreed to in writing, software\n    distributed under the License is distributed on an \"AS IS\" BASIS,\n    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n    See the License for the specific language governing permissions and\n    limitations under the License.","ref":"license.html#appendix-how-to-apply-the-apache-license-to-your-work"}],"content_type":"text/plain","producer":{"name":"ex_doc","version":[48,46,51,50,46,50]}}